<?php
/**
 * The main plugin class
 *
 * @package plugins
 */

/**
 * Class BPMVP_Plugin
 */
class BPMVP_Plugin {

	/**
	 * Whether we run in debugging mode.
	 *
	 * @var  bool
	 */
	protected static $debug = NULL;

	/**
	 * Plugin instance.
	 *
	 * @see   get_instance()
	 * @var  object
	 */
	protected static $instance = NULL;

	/**
	 * URL to this plugin's directory.
	 *
	 * @var  string
	 */
	public $plugin_url = '';

	/**
	 * Path to this plugin's directory.
	 *
	 * @var  string
	 */
	public $plugin_path = '';

	/**
	 * The API Object.
	 *
	 * @var object
	 */
	private $api = NULL;

	/**
	 * The validator object.
	 *
	 * @var object
	 */
	private $validator = NULL;

	/**
	 * The options
	 *
	 * @var array
	 */
	private $options = array();

	/**
	 * Access this plugin’s working instance
	 *
	 * @wp-hook plugins_loaded
	 * @return  object of this class
	 */
	public static function get_instance() {

		if ( NULL === self::$debug ) {
			self::$debug = ( defined( 'WP_DEBUG' ) && WP_DEBUG ) ? TRUE : FALSE;
		}

		NULL === self::$instance and self::$instance = new self;

		return self::$instance;
	}

	/**
	 * Used for regular plugin work.
	 *
	 * @wp-hook  plugins_loaded
	 * @return   void
	 */
	public function plugin_setup() {

		$this->plugin_url	= plugins_url( '/', dirname( __FILE__ ) );
		$this->plugin_path   = 'email-validator-by-byteplant/';
		$this->load_language( 'email-validator-by-byteplant' );

		// Load Options.
		$this->set_options( get_option( 'bpmvp_settings', array() ) );

		// Load the API.
		require_once( dirname( __FILE__ ) . '/API/class-bpmvp-api.php' );
		require_once( dirname( __FILE__ ) . '/API/class-bpmvp-email-validator.php' );
		$this->api = new BPMVP_API();
		$this->api->set_apikey( $this->get_option( 'bpmvp_api_key' ) );
		$this->validator = new BPMVP_Email_Validator();
		$this->validator->set_api( $this->api );
		if ( 0 === (int) $this->get_option( 'bpmvp_rc305_check' ) ) {
			$this->validator->add_status_valid('305');
		}
		if ( 0 === (int) $this->get_option( 'bpmvp_rc308_check' ) ) {
			$this->validator->add_status_valid('308');
		}
		$this->validator->set_blacklist( $this->get_option( 'bpmvp_blacklist' ) );

		// Load the Ajax Interface.
		require_once( dirname( __FILE__ ) . '/AJAX/class-bpmvp-ajax-handler.php' );
		require_once( dirname( __FILE__ ) . '/AJAX/class-bpmvp-ajax-validate-email.php' );
		$ajax_handler = new BPMVP_Ajax_Handler();
		$ajax = new BPMVP_Ajax_Validate_Email();
		$ajax->set_validator( $this->validator );
		$ajax->set_handler( $ajax_handler );

		if ( 1 === (int) $this->get_option( 'bpmvp_is_email_check' ) ) {
			require_once( dirname( __FILE__ ) . '/Checks/class-bpmvp-is-email.php' );
			$email_check = new BPMVP_is_email();
			$email_check->set_validator( $this->validator );
			$email_check->setup();
		}

		if ( 1 === (int) $this->get_option( 'bpmvp_ninja_check' ) ) {
			require_once( dirname( __FILE__ ) . '/Checks/class-bpmvp-ninja-forms.php' );
			$email_check = new BPMVP_NinjaForms();
			$email_check->set_validator( $this->validator );
			$email_check->setup();
		}

		if ( 1 === (int) $this->get_option( 'bpmvp_comments_check' ) ) {
			require_once( dirname( __FILE__ ) . '/Checks/class-bpmvp-on-comment.php' );
			$comment_check = new BPMVP_On_Comment();
			$comment_check->set_validator( $this->validator );
			$comment_check->setup();
		}

		if ( 1 === (int) $this->get_option( 'bpmvp_reg_check' ) ) {
			require_once( dirname( __FILE__ ) . '/Checks/class-bpmvp-on-registration.php' );
			$comment_check = new BPMVP_On_Registration();
			$comment_check->set_validator( $this->validator );
			$comment_check->setup();
		}

		/**
		 * Filters whether the ajax call is only for logged in users. Default: FALSE.
		 *
		 * @param bool
		 */
		$ajax_is_private = (bool) apply_filters( 'bpmvp_api_is_private', TRUE );
		$ajax->set_private( $ajax_is_private );
		$ajax->register();

		add_action( 'wp_enqueue_scripts', array( $this, 'register_script' ) );

		if ( is_admin() ) {
			// Load the Admin Interface.
			require_once( dirname( __FILE__ ) . '/class-bpmvp-admin.php' );
			$admin = BPMVP_Admin::get_instance();
			$admin->load();
		}
	}

	/**
	 * Whether we are in debug mode.
	 *
	 * @return bool
	 */
	public function is_debug() {

		return self::$debug;
	}

	/**
	 * Register the Javascript.
	 */
	public function register_script() {

		// Minimize the script on production site.
		$min = '';
		if ( ! self::$debug ) {
			$min = '.min';
		}
		wp_register_script('bpmvp_frontend_script', $this->get_plugin_url() . 'assets/js/bpmvp_form_script' . $min . '.js', array( 'jquery', 'underscore' ), '1.0', TRUE );

		$js_vars = $this->js_localization();

		// The HTML templates.
		$js_vars['tpl']    = '<span class="bpmvp-error"><%- status %> <%- mail %></span>';

		wp_localize_script( 'bpmvp_frontend_script', 'bpmvp', $js_vars);
	}

	/**
	 * Enqueue Script on frontend.
	 *
	 * @see bpmvp_activate_third_party()
	 */
	public function enqueue_frontend() {
		wp_enqueue_script( 'bpmvp_frontend_script' );
	}

	/**
	 * Add footer styles on frontend
	 */
	public function footer_styles() {
		?><style>.bpmvp-error {color: #f00;}</style><?php
	}
	/**
	 * Get the plugin url
	 *
	 * @return string plugin url.
	 */
	public function get_plugin_url() {

		return $this->plugin_url;
	}

	/**
	 * Get the plugin url
	 *
	 * @return string plugin path.
	 */
	public function get_plugin_path() {

		return $this->plugin_path;
	}

	/**
	 * Localized Javascript Variables.
	 *
	 * @return array
	 **/
	public function js_localization() {

		return array(
			'AJAX_URL' => esc_js( admin_url( 'admin-ajax.php' ) ),
			'nonce'    => esc_js( wp_create_nonce( 'validate-email' ) ),

			'200'      => __( 'OK - Valid Address', 'email-validator-by-byteplant' ),
			'207'      => __( 'OK - Catch-All Active', 'email-validator-by-byteplant' ),
			'215'      => __( 'OK - Catch-All Test Delayed', 'email-validator-by-byteplant' ),

			'302'      => __( 'Local Address', 'email-validator-by-byteplant' ),
			'303'      => __( 'IP Address Literal', 'email-validator-by-byteplant' ),
			'305'      => __( 'Disposable Address', 'email-validator-by-byteplant' ),
			'308'      => __( 'Role Address', 'email-validator-by-byteplant' ),
			'313'      => __( 'Server Unavailable', 'email-validator-by-byteplant' ),
			'314'      => __( 'Address Unavailable', 'email-validator-by-byteplant' ),
			'316'      => __( 'Duplicate Address', 'email-validator-by-byteplant' ),
			'317'      => __( 'Server Reject', 'email-validator-by-byteplant' ),

			'401'      => __( 'Bad Address', 'email-validator-by-byteplant' ),
			'404'      => __( 'Domain Not Fully Qualified', 'email-validator-by-byteplant' ),
			'406'      => __( 'MX Lookup Error', 'email-validator-by-byteplant' ),
			'409'      => __( 'No-Reply Address', 'email-validator-by-byteplant' ),
			'410'      => __( 'Address Rejected', 'email-validator-by-byteplant' ),
			'413'      => __( 'Server Unavailable', 'email-validator-by-byteplant' ),
			'414'      => __( 'Address Unavailable', 'email-validator-by-byteplant' ),
			'420'      => __( 'Domain Name Misspelled', 'email-validator-by-byteplant' ),

			'114'      => __( 'Validation Delayed', 'email-validator-by-byteplant' ),
			'118'      => __( 'Rate Limit Exceeded', 'email-validator-by-byteplant' ),
			'119'      => __( 'API Key Invalid or Depleted', 'email-validator-by-byteplant' ),
			'121'      => __( 'Task Accepted', 'email-validator-by-byteplant' ),

			'800'      => __( 'Email Address Missing', 'email-validator-by-byteplant' ),
			'801'      => __( 'Service Unavailable', 'email-validator-by-byteplant' ),
		);
	}

	/**
	 * Get the options.
	 *
	 * @return array
	 */
	public function get_options() {

		return $this->options;
	}

	/**
	 * Set the options.
	 *
	 * @param array $options The options.
	 *
	 * @return array
	 */
	public function set_options( $options ) {

		$defaults = array(
			'bpmvp_api_key' => '',
		);
		$this->options = wp_parse_args( (array) $options, $defaults );
		return $this->get_options();
	}

	/**
	 * Return a single option.
	 *
	 * @param string $option_key The option key.
	 *
	 * @return mixed|null
	 */
	public function get_option( $option_key ) {

		return ( ! empty( $this->options[ $option_key ] ) ) ? $this->options[ $option_key ] : NULL;
	}



	/**
	 * Empty and protected constructor.
	 */
	protected function __construct() {}

	/**
	 * Empty and private clone.
	 */
	private function __clone(){}


	/**
	 * Loads translation file.
	 *
	 * Accessible to other classes to load different language files (admin and
	 * front-end for example).
	 *
	 * @wp-hook init
	 * @param   string $domain The plugins domain.
	 * @return  void
	 */
	public function load_language( $domain ) {

		load_plugin_textdomain(
			$domain,
			FALSE,
			$this->plugin_path . 'languages'
		);
	}
}
